@@ -5,19 +5,24 @@ use warnings;
 use utf8;
 
 use Tickit;
-use Tickit::Widgets qw( SegmentDisplay Box VBox HBox );
+use Tickit::Widgets 0.30 qw(
+   SegmentDisplay
+   Box=0.53
+   VBox=0.48
+   HBox=0.48
+);
+use Tickit::Widget::Box 0.53;
 
 my $tickit = Tickit->new(
    root => Tickit::Widget::VBox->new(
       spacing => 1,
-      children => [
-         my $unit_box = Tickit::Widget::HBox->new(
-            spacing => 2,
-         ),
-         my $prefix_box = Tickit::Widget::HBox->new(
-            spacing => 2,
-         ),
-      ],
+   )->add_children(
+      my $unit_box = Tickit::Widget::HBox->new(
+         spacing => 2,
+      ),
+      my $prefix_box = Tickit::Widget::HBox->new(
+         spacing => 2,
+      ),
    )
 );
 
@@ -27,14 +32,15 @@ sub _add
 
    $hbox->add(
       Tickit::Widget::Box->new(
-         child => Tickit::Widget::SegmentDisplay->new(
+         child_lines => 11,
+         child_cols  => 14,
+      )->set_child(
+         Tickit::Widget::SegmentDisplay->new(
             type => 'symb',
             value => $unit,
             use_halfline => 1,
             thickness    => 1,
          ),
-         child_lines => 11,
-         child_cols  => 14,
       )
    );
 }
